@@ -215,6 +215,7 @@ read_next_line(grn_ctx *ctx, grn_obj *buf)
     rc = line_editor_fgets(ctx, buf);
 #else
     fprintf(stderr, "> ");
+    fflush(stderr);
     rc = grn_text_fgets(ctx, buf, stdin);
 #endif
   } else {
@@ -657,6 +658,7 @@ h_output(grn_ctx *ctx, int flags, void *arg)
   ht_context *hc = (ht_context *)arg;
   grn_sock fd = hc->msg->u.fd;
   grn_obj header, head, foot, *outbuf = ctx->impl->outbuf;
+  grn_bool should_return_body = (hc->msg->header.qtype == 'G');
   if (!(flags & GRN_CTX_TAIL)) { return; }
   GRN_TEXT_INIT(&header, 0);
   GRN_TEXT_INIT(&head, 0);
@@ -687,18 +689,25 @@ h_output(grn_ctx *ctx, int flags, void *arg)
   {
     ssize_t ret, len;
 #ifdef WIN32
+    int n_buffers;
     WSABUF wsabufs[4];
     wsabufs[0].buf = GRN_TEXT_VALUE(&header);
     wsabufs[0].len = GRN_TEXT_LEN(&header);
-    wsabufs[1].buf = GRN_TEXT_VALUE(&head);
-    wsabufs[1].len = GRN_TEXT_LEN(&head);
-    wsabufs[2].buf = GRN_TEXT_VALUE(outbuf);
-    wsabufs[2].len = GRN_TEXT_LEN(outbuf);
-    wsabufs[3].buf = GRN_TEXT_VALUE(&foot);
-    wsabufs[3].len = GRN_TEXT_LEN(&foot);
+    n_buffers = 1;
+    len = GRN_TEXT_LEN(&header);
+    if (should_return_body) {
+      wsabufs[1].buf = GRN_TEXT_VALUE(&head);
+      wsabufs[1].len = GRN_TEXT_LEN(&head);
+      wsabufs[2].buf = GRN_TEXT_VALUE(outbuf);
+      wsabufs[2].len = GRN_TEXT_LEN(outbuf);
+      wsabufs[3].buf = GRN_TEXT_VALUE(&foot);
+      wsabufs[3].len = GRN_TEXT_LEN(&foot);
+      n_buffers += 3;
+      len += GRN_TEXT_LEN(&head) + GRN_TEXT_LEN(outbuf) + GRN_TEXT_LEN(&foot);
+    }
     {
       DWORD sent;
-      if (WSASend(fd, wsabufs, 4, &sent, 0, NULL, NULL) == SOCKET_ERROR) {
+      if (WSASend(fd, wsabufs, n_buffers, &sent, 0, NULL, NULL) == SOCKET_ERROR) {
         SERR("WSASend");
       }
       ret = sent;
@@ -709,24 +718,27 @@ h_output(grn_ctx *ctx, int flags, void *arg)
     msg.msg_name = NULL;
     msg.msg_namelen = 0;
     msg.msg_iov = msg_iov;
-    msg.msg_iovlen = 4;
+    msg.msg_iovlen = 1;
     msg.msg_control = NULL;
     msg.msg_controllen = 0;
     msg.msg_flags = 0;
     msg_iov[0].iov_base = GRN_TEXT_VALUE(&header);
     msg_iov[0].iov_len = GRN_TEXT_LEN(&header);
-    msg_iov[1].iov_base = GRN_TEXT_VALUE(&head);
-    msg_iov[1].iov_len = GRN_TEXT_LEN(&head);
-    msg_iov[2].iov_base = GRN_TEXT_VALUE(outbuf);
-    msg_iov[2].iov_len = GRN_TEXT_LEN(outbuf);
-    msg_iov[3].iov_base = GRN_TEXT_VALUE(&foot);
-    msg_iov[3].iov_len = GRN_TEXT_LEN(&foot);
+    len = GRN_TEXT_LEN(&header);
+    if (should_return_body) {
+      msg_iov[1].iov_base = GRN_TEXT_VALUE(&head);
+      msg_iov[1].iov_len = GRN_TEXT_LEN(&head);
+      msg_iov[2].iov_base = GRN_TEXT_VALUE(outbuf);
+      msg_iov[2].iov_len = GRN_TEXT_LEN(outbuf);
+      msg_iov[3].iov_base = GRN_TEXT_VALUE(&foot);
+      msg_iov[3].iov_len = GRN_TEXT_LEN(&foot);
+      msg.msg_iovlen += 3;
+      len += GRN_TEXT_LEN(&head) + GRN_TEXT_LEN(outbuf) + GRN_TEXT_LEN(&foot);
+    }
     if ((ret = sendmsg(fd, &msg, MSG_NOSIGNAL)) == -1) {
       SERR("sendmsg");
     }
 #endif /* WIN32 */
-    len = GRN_TEXT_LEN(&header) + GRN_TEXT_LEN(&head) +
-      GRN_TEXT_LEN(outbuf) + GRN_TEXT_LEN(&foot);
     if (ret != len) {
       GRN_LOG(&grn_gctx, GRN_LOG_NOTICE,
               "couldn't send all data (%" GRN_FMT_LLD "/%" GRN_FMT_LLD ")",
@@ -1061,6 +1073,7 @@ do_htreq(grn_ctx *ctx, grn_msg *msg)
   grn_com_header *header = &msg->header;
   switch (header->qtype) {
   case 'G' : /* GET */
+  case 'H' : /* HEAD */
     do_htreq_get(ctx, msg);
     break;
   case 'P' : /* POST */
@@ -1900,18 +1913,18 @@ g_server(char *path)
 }
 
 enum {
-  mode_alone = 0,
-  mode_client,
-  mode_daemon,
-  mode_server,
-  mode_usage,
-  mode_version,
-  mode_config,
-  mode_error
+  ACTION_USAGE = 1,
+  ACTION_VERSION,
+  ACTION_SHOW_CONFIG,
+  ACTION_ERROR
 };
 
-#define MODE_MASK   0x007f
-#define MODE_NEW_DB 0x0100
+#define ACTION_MASK       (0x0f)
+#define FLAG_MODE_ALONE      (1 << 4)
+#define FLAG_MODE_CLIENT     (1 << 5)
+#define FLAG_MODE_DAEMON     (1 << 6)
+#define FLAG_MODE_SERVER     (1 << 7)
+#define FLAG_NEW_DB     (1 << 8)
 
 static uint32_t
 get_core_number(void)
@@ -2132,7 +2145,6 @@ static const int default_http_port = DEFAULT_HTTP_PORT;
 static const int default_gqtp_port = DEFAULT_GQTP_PORT;
 static grn_encoding default_encoding = GRN_ENC_DEFAULT;
 static uint32_t default_max_num_threads = DEFAULT_MAX_NFTHREADS;
-static const int default_mode = mode_alone;
 static const int default_log_level = GRN_LOG_DEFAULT_LEVEL;
 static const char * const default_protocol = "gqtp";
 static const char *default_hostname = "localhost";
@@ -2406,26 +2418,27 @@ main(int argc, char **argv)
     *working_directory_arg = NULL;
   const char *config_path = NULL;
   int exit_code = EXIT_SUCCESS;
-  int i, mode = mode_alone;
+  int i;
+  int flags = 0;
   uint32_t cache_limit = 0;
   static grn_str_getopt_opt opts[] = {
     {'p', "port", NULL, 0, GETOPT_OP_NONE},
     {'e', "encoding", NULL, 0, GETOPT_OP_NONE},
     {'t', "max-threads", NULL, 0, GETOPT_OP_NONE},
-    {'h', "help", NULL, mode_usage, GETOPT_OP_UPDATE},
-    {'c', NULL, NULL, mode_client, GETOPT_OP_UPDATE},
-    {'d', NULL, NULL, mode_daemon, GETOPT_OP_UPDATE},
-    {'s', NULL, NULL, mode_server, GETOPT_OP_UPDATE},
+    {'h', "help", NULL, ACTION_USAGE, GETOPT_OP_UPDATE},
+    {'c', NULL, NULL, FLAG_MODE_CLIENT, GETOPT_OP_ON},
+    {'d', NULL, NULL, FLAG_MODE_DAEMON, GETOPT_OP_ON},
+    {'s', NULL, NULL, FLAG_MODE_SERVER, GETOPT_OP_ON},
     {'l', "log-level", NULL, 0, GETOPT_OP_NONE},
     {'i', "server-id", NULL, 0, GETOPT_OP_NONE},
-    {'n', NULL, NULL, MODE_NEW_DB, GETOPT_OP_ON},
+    {'n', NULL, NULL, FLAG_NEW_DB, GETOPT_OP_ON},
     {'\0', "protocol", NULL, 0, GETOPT_OP_NONE},
-    {'\0', "version", NULL, mode_version, GETOPT_OP_UPDATE},
+    {'\0', "version", NULL, ACTION_VERSION, GETOPT_OP_UPDATE},
     {'\0', "log-path", NULL, 0, GETOPT_OP_NONE},
     {'\0', "query-log-path", NULL, 0, GETOPT_OP_NONE},
     {'\0', "pid-path", NULL, 0, GETOPT_OP_NONE},
     {'\0', "config-path", NULL, 0, GETOPT_OP_NONE},
-    {'\0', "show-config", NULL, mode_config, GETOPT_OP_UPDATE},
+    {'\0', "show-config", NULL, ACTION_SHOW_CONFIG, GETOPT_OP_UPDATE},
     {'\0', "cache-limit", NULL, 0, GETOPT_OP_NONE},
     {'\0', "file", NULL, 0, GETOPT_OP_NONE},
     {'\0', "document-root", NULL, 0, GETOPT_OP_NONE},
@@ -2462,14 +2475,14 @@ main(int argc, char **argv)
   init_default_settings();
 
   /* only for parsing --config-path. */
-  i = grn_str_getopt(argc, argv, opts, &mode);
+  i = grn_str_getopt(argc, argv, opts, &flags);
   if (i < 0) {
     show_usage(stderr);
     return EXIT_FAILURE;
   }
 
   if (config_path) {
-    const config_file_status status = config_file_load(config_path, opts, &mode);
+    const config_file_status status = config_file_load(config_path, opts, &flags);
     if (status == CONFIG_FILE_FOPEN_ERROR) {
       fprintf(stderr, "%s: can't open config file: %s (%s)\n",
               argv[0], config_path, strerror(errno));
@@ -2482,7 +2495,7 @@ main(int argc, char **argv)
     }
   } else if (*default_config_path) {
     const config_file_status status =
-        config_file_load(default_config_path, opts, &mode);
+        config_file_load(default_config_path, opts, &flags);
     if (status != CONFIG_FILE_SUCCESS && status != CONFIG_FILE_FOPEN_ERROR) {
       fprintf(stderr, "%s: failed to parse config file: %s (%s)\n",
               argv[0], default_config_path,
@@ -2500,22 +2513,21 @@ main(int argc, char **argv)
   }
 
   /* ignore mode option in config file */
-  mode = (mode == mode_error) ? default_mode :
-    ((mode & ~MODE_MASK) | default_mode);
+  flags = (flags == ACTION_ERROR) ? 0 : (flags & ~ACTION_MASK);
 
-  i = grn_str_getopt(argc, argv, opts, &mode);
-  if (i < 0) { mode = mode_error; }
-  switch (mode & MODE_MASK) {
-  case mode_version :
+  i = grn_str_getopt(argc, argv, opts, &flags);
+  if (i < 0) { flags = ACTION_ERROR; }
+  switch (flags & ACTION_MASK) {
+  case ACTION_VERSION :
     show_version();
     return EXIT_SUCCESS;
-  case mode_usage :
+  case ACTION_USAGE :
     show_usage(output);
     return EXIT_SUCCESS;
-  case mode_config :
-    show_config(output, opts, mode & ~MODE_MASK);
+  case ACTION_SHOW_CONFIG :
+    show_config(output, opts, flags & ~ACTION_MASK);
     return EXIT_SUCCESS;
-  case mode_error :
+  case ACTION_ERROR :
     show_usage(stderr);
     return EXIT_FAILURE;
   }
@@ -2796,23 +2808,14 @@ main(int argc, char **argv)
     grn_cache_set_max_n_entries(&grn_gctx, cache, cache_limit);
   }
 
-  newdb = (mode & MODE_NEW_DB);
-  switch (mode & MODE_MASK) {
-  case mode_alone :
-    exit_code = do_alone(argc - i, argv + i);
-    break;
-  case mode_client :
+  newdb = (flags & FLAG_NEW_DB);
+  is_daemon_mode = (flags & FLAG_MODE_DAEMON);
+  if (flags & FLAG_MODE_CLIENT) {
     exit_code = do_client(argc - i, argv + i);
-    break;
-  case mode_daemon :
-    is_daemon_mode = GRN_TRUE;
-    /* fallthru */
-  case mode_server :
+  } else if (is_daemon_mode || (flags & FLAG_MODE_SERVER)) {
     exit_code = do_server(argc > i ? argv[i] : NULL);
-    break;
-  default:
-    exit_code = EXIT_FAILURE;
-    break;
+  } else {
+    exit_code = do_alone(argc - i, argv + i);
   }
 
 #ifdef GRN_WITH_LIBEDIT
